function [n1,n2,n3,k1,k2,k3,e_xx,e_xy,e_xz,e_yy,e_yz,e_zz]=dispersion_model_triclinic(nu,gamm,Sk,phi,theta,epsilxx,epsilxy,epsilxz,epsilyy,epsilyz,epsilzz,p)
% dispersion_model_triclinic: Mayerhofer et al.
% calculates dielectric tenso st, omega, chi, psi)
% gamm: damping parameter
% Sk: bandwidth parameter
% phi: azimuthal angle
% theta: angle w.r.t. Z-axis
% epsil: bulk dialectric constants

%global pool_size;
%ps=pool_size;

v=p(:,1); omega=p(:,2); chi=p(:,3); psi=p(:,4);

% make nu, fourpr, gamm be 1xN 
nu=nu(:)'; Sk=Sk(:)'; gamm=gamm(:)'; phi=phi(:)'; theta=theta(:)';

% duplicate v etc. times the number of oscillators
v=repmat(v,1,length(nu));
omega=repmat(omega,1,length(nu));
chi=repmat(chi,1,length(nu));
psi=repmat(psi,1,length(nu));

% duplicate oscillator parameters times the number of wavelengths
nu=repmat(nu,length(v),1);
gamm=repmat(gamm,length(v),1);
Sk=repmat(Sk,length(v),1);
phi=repmat(phi,length(v),1);
theta=repmat(theta,length(v),1);

%calculate dielectric tensor
uxx=((sind(theta)).^2).*((cosd(phi)).^2);
uxy=((sind(theta)).^2).*(cosd(phi)).*(sind(phi));
uxz=(sind(theta)).*(cosd(theta)).*(cosd(phi));
uyy=((sind(theta)).^2).*((sind(phi)).^2);
uyz=(sind(theta)).*(cosd(theta)).*(sind(phi));
uzz=(cosd(theta)).^2;

denom=(nu.^2-v.^2)-(sqrt(-1))*gamm.*v;

L_k_xx=uxx.*Sk./denom;  
L_k_xy=uxy.*Sk./denom; 
L_k_xz=uxz.*Sk./denom;
L_k_yy=uyy.*Sk./denom;
L_k_yz=uyz.*Sk./denom;
L_k_zz=uzz.*Sk./denom;

e_123=zeros(3,3,length(v));

e_123(1,1,:)=epsilxx + sum(L_k_xx,2);  
e_123(1,2,:)=epsilxy + sum(L_k_xy,2); e_123(2,1,:)=e_123(1,2,:);
e_123(1,3,:)=epsilxz + sum(L_k_xz,2); e_123(3,1,:)=e_123(1,3,:);
e_123(2,2,:)=epsilyy + sum(L_k_yy,2); 
e_123(2,3,:)=epsilyz + sum(L_k_yz,2); e_123(3,2,:)=e_123(2,3,:);
e_123(3,3,:)=epsilzz + sum(L_k_zz,2);

%rotate dielectric tensor to match lab coordinate system
%{
poolobj = gcp('nocreate'); % If no pool, do not create new one.
if isempty(poolobj)
    poolsize = 0;
else
    poolsize = poolobj.NumWorkers;
end

if poolsize == 0
    parpool(ps);
end
%}
lam=zeros(3,length(v));
parfor j=1:length(v)
  A=zeros(3,3);

  A(1,1)=(cosd(chi(j,1))).*(cosd(omega(j,1))).*(cosd(psi(j,1)))-(sind(omega(j,1))).*(sind(psi(j,1)));
  A(1,2)=(cosd(chi(j,1))).*(cosd(psi(j,1))).*(sind(omega(j,1)))+(cosd(omega(j,1))).*(sind(psi(j,1)));
  A(1,3)=-(cosd(psi(j,1))).*(sind(chi(j,1)));
  A(2,1)=-(cosd(psi(j,1))).*(sind(omega(j,1)))-(cosd(chi(j,1))).*(cosd(omega(j,1))).*(sind(psi(j,1)));
  A(2,2)=(cosd(omega(j,1))).*(cosd(psi(j,1)))-(cosd(chi(j,1))).*(sind(omega(j,1))).*(sind(psi(j,1)));
  A(2,3)=(sind(chi(j,1))).*(sind(psi(j,1)));
  A(3,1)=(cosd(omega(j,1))).*(sind(chi(j,1)));
  A(3,2)=(sind(chi(j,1))).*(sind(omega(j,1)));
  A(3,3)=cosd(chi(j,1));

  A_inv=pinv(A);

  e_xyz=A*e_123(:,:,j)*A_inv;
  
  e_xx(j,1,1)=e_xyz(1,1);
  e_xy(j,1,1)=e_xyz(1,2);
  e_xz(j,1,1)=e_xyz(1,3);
  e_yy(j,1,1)=e_xyz(2,2);
  e_yz(j,1,1)=e_xyz(2,3);
  e_zz(j,1,1)=e_xyz(3,3);
  
  lam(:,j)=eig(e_123(:,:,j));
end

% calculate optical constants (n's + k's)
m1sq=zeros(size(lam(1,:))); m2sq=zeros(size(lam(1,:))); m3sq=zeros(size(lam(1,:))); 
m1sq(1)=lam(1,1); m2sq(1)=lam(2,1); m3sq(1)=lam(3,1);

% enforce continuity
for j=2:length(v)  
    dist=[0 0 0];
    dist(1)=abs(lam(1,j)-m1sq(j-1)); dist(2)=abs(lam(2,j)-m1sq(j-1)); dist(3)=abs(lam(3,j)-m1sq(1,j-1));
    [~,I1]=min(dist); 
    m1sq(j)=lam(I1,j);
    dist(1)=abs(lam(1,j)-m2sq(j-1)); dist(2)=abs(lam(2,j)-m2sq(j-1)); dist(3)=abs(lam(3,j)-m2sq(j-1));
    dist(I1)=1000;
    [~,I2]=min(dist);
    m2sq(j)=lam(I2,j);
    dist(1)=abs(lam(1,j)-m3sq(j-1)); dist(2)=abs(lam(2,j)-m3sq(j-1)); dist(3)=abs(lam(3,j)-m3sq(j-1));
    dist(I1)=1000; dist(I2)=1000;
    [~,I3]=min(dist);
    m3sq(j)=lam(I3,j);
end

m1=sqrt(m1sq); m2=sqrt(m2sq); m3=sqrt(m3sq);

n1=real(m1); k1=imag(m1); 
n2=real(m2); k2=imag(m2); 
n3=real(m3); k3=imag(m3);

n1=n1'; n2=n2'; n3=n3';
k1=k1'; k2=k2'; k3=k3';
