function R_sbu=fresnel_triclinic(e_xx,e_xy,e_xz,e_yy,e_yz,e_zz,p)
% fresnel_triclinic: calculates reflectance from dielectric tensor using
% fresnel model for incidence angle of alpha degrees
%global pool_size;
%ps=pool_size;

global incidence_angle;
alpha=incidence_angle;

k_y=sind(alpha);
polarisation=p(:,5:6);

%make e's be 1x1xv
e_xx=reshape(e_xx, [1 1 size(e_xx,1)]); e_xy=reshape(e_xy, [1 1 size(e_xy,1)]); e_xz=reshape(e_xz, [1 1 size(e_xz,1)]);
e_yy=reshape(e_yy, [1 1 size(e_yy,1)]); e_yz=reshape(e_yz, [1 1 size(e_yz,1)]); e_zz=reshape(e_zz, [1 1 size(e_zz,1)]);
len_v=size(e_xx,3);

%dynamical matrix of incident meduim (air, n=1)
D_0_inv=[.5 .5/(cosd(alpha)) 0 0; .5 -.5/(cosd(alpha)) 0 0; 0 0 .5/(cosd(alpha)) .5; 0 0 .5/(cosd(alpha)) -.5];

%calculate M matrix
M = zeros(4,4,len_v);
delta=zeros(4,4,len_v);

delta(1,2,:)=1;
delta(2,1,:)=-(k_y^2)+e_xx-(e_xz.^2)./e_zz; delta(2,3,:)=e_xy-e_xz.*e_yz./e_zz; delta(2,4,:)=-k_y*e_xz./e_zz;
delta(3,1,:)=-k_y*e_xz./e_zz; delta(3,3,:)=-k_y*e_yz./e_zz; delta(3,4,:)=1-(k_y^2)./e_zz;
delta(4,1,:)=e_xy-e_yz.*e_xz./e_zz; delta(4,3,:)=e_yy-(e_yz.^2)./e_zz; delta(4,4,:)=-k_y*e_yz./e_zz;

gammpos_old=[1 3];
for j=1:1:len_v
    clear gammpos;
    D_1=zeros(4);
    [D,Gamma]=eig(delta(:,:,j));% this can be done algebraically, and I have
                                % the eigenvalues of delta written out in
                                % Mathematica
                                           
    l=1;
    for k=1:1:4
        if and(real(Gamma(k,k))>0, imag(Gamma(k,k))>0)  %there should be two
                                                        %positive values, but that's 
                                                        %not always the
                                                        %case while the fit
                                                        %is still
                                                        %converging
            gammpos(l)=k;
            l=2;
        end
    end
    
    if length(gammpos)<2
                    gammpos(1)=gammpos_old(1);
                    gammpos(2)=gammpos_old(2);
    end
    gammpos_old(1)=gammpos(1); gammpos_old(2)=gammpos(2);
    
    if (abs(Gamma(gammpos(1),gammpos(1)))<=abs(Gamma(gammpos(2),gammpos(2)))) 
        D_1(:,1)=D(:,gammpos(1)); D_1(:,3)=D(:,gammpos(2));
    else
        D_1(:,1)=D(:,gammpos(2)); D_1(:,3)=D(:,gammpos(1));
    end

    M(:,:,j)=D_0_inv*D_1;
end

% calculate reflectance coefficients (s=s-polarised, p=p-polarised, 1st
% letter=polariser, 2nd letter=analyser)
%{
poolobj = gcp('nocreate'); % If no pool, do not create new one.
if isempty(poolobj)
    poolsize = 0;
else
    poolsize = poolobj.NumWorkers;
end

if poolsize == 0
    parpool(ps);
end
%}
spmd
denom=M(1,1,:).*M(3,3,:)-M(1,3,:).*M(3,1,:);

r_ss=(M(2,1,:).*M(3,3,:)-M(2,3,:).*M(3,1,:))./denom;
r_ss=reshape(r_ss, [size(r_ss,3) 1]);
r_sp=(M(4,1,:).*M(3,3,:)-M(4,3,:).*M(3,1,:))./denom;
r_sp=reshape(r_sp, [size(r_sp,3) 1]);
r_ps=(M(1,1,:).*M(2,3,:)-M(2,1,:).*M(1,3,:))./denom;
r_ps=reshape(r_ps, [size(r_ps,3) 1]);
r_pp=(M(1,1,:).*M(4,3,:)-M(4,1,:).*M(1,3,:))./denom;
r_pp=reshape(r_pp, [size(r_pp,3) 1]);

% calculate reflectance
R_s=(abs(r_ss)).^2+(abs(r_sp)).^2;
R_p=(abs(r_ps)).^2+(abs(r_pp)).^2;
R=[R_s R_p];
    
R_sbu_dist=sum(R.*polarisation,2);
end
R_sbu=gather(R_sbu_dist);
R_sbu=R_sbu{1};

