% This script derives oriented optical constants from reflectance data. 
% 
% The functions called by the main script are:
% disp_model_wrap_oblique.m  :Wrapper function needed to ease use of
%                             lsqcurvefit. 
% disp_model_wrap_final.m    :Wrapper function to output final (n,k) values 
%                             and reflectance spectra and print these
%                             values to textfiles.
% Wrapper functions call the following:
% dispersion_model_oblique.m :Actual dispersion model called by the
%                             wrapper function. Outputs optical constants 
%                             (n,k).
% fresnel_oblique.m          :Fresnel equations for non-normal incidence. Calculates 
%                             reflectance from optical constants. 
% isint.m                    :Short boolean function that gives 'Yes' if
%                             the input is an integer. 


%% Section 1

%first decide on numer of workers for parallel loops
 %global pool_size
 %pool_size=52;
 
% Read in reflectance data and trim to a range of wavenumbers.
[v1,Rp_000]=textread('./Labradorite_FTIR_Melinda/Rp000.csv','%f %f','delimiter',',');
[v2,Rs_000]=textread('./Labradorite_FTIR_Melinda/Rs000.csv','%f %f','delimiter',',');
[v3,Rp_4500]=textread('./Labradorite_FTIR_Melinda/Rp4500.csv','%f %f','delimiter',',');
[v4,Rs_4500]=textread('./Labradorite_FTIR_Melinda/Rs4500.csv','%f %f','delimiter',',');

[v5,Rp_0900]=textread('./Labradorite_FTIR_Melinda/Rp0900.csv','%f %f','delimiter',',');
[v6,Rs_0900]=textread('./Labradorite_FTIR_Melinda/Rs0900.csv','%f %f','delimiter',',');
[v7,Rp_09045]=textread('./Labradorite_FTIR_Melinda/Rp09045.csv','%f %f','delimiter',',');
[v8,Rs_09045]=textread('./Labradorite_FTIR_Melinda/Rs09045.csv','%f %f','delimiter',',');

[v9,Rp_909090]=textread('./Labradorite_FTIR_Melinda/Rp909090.csv','%f %f','delimiter',',');
[v10,Rs_909090]=textread('./Labradorite_FTIR_Melinda/Rs909090.csv','%f %f','delimiter',',');
[v11,Rp_909045]=textread('./Labradorite_FTIR_Melinda/Rp909045.csv','%f %f','delimiter',',');
[v12,Rs_909045]=textread('./Labradorite_FTIR_Melinda/Rs909045.csv','%f %f','delimiter',',');

% Clip reflectance data to desired wavenumber range.
lo_wn=225; % set lower bound
up_wn=2000; % set upper bound

Rp_000=Rp_000(find(v1>lo_wn));       Rs_000=Rs_000(find(v2>lo_wn));        Rp_4500=Rp_4500(find(v3>lo_wn));      Rs_4500=Rs_4500(find(v4>lo_wn)); 
Rp_0900=Rp_0900(find(v5>lo_wn));     Rs_0900=Rs_0900(find(v6>lo_wn));      Rp_09045=Rp_09045(find(v7>lo_wn));    Rs_09045=Rs_09045(find(v8>lo_wn));
Rp_909090=Rp_909090(find(v9>lo_wn)); Rs_909090=Rs_909090(find(v10>lo_wn)); Rp_909045=Rp_909045(find(v11>lo_wn)); Rs_909045=Rs_909045(find(v12>lo_wn));

v1=v1(find(v1>lo_wn)); v2=v2(find(v2>lo_wn));    v3=v3(find(v3>lo_wn));    v4=v4(find(v4>lo_wn)); 
v5=v5(find(v5>lo_wn)); v6=v6(find(v6>lo_wn));    v7=v7(find(v7>lo_wn));    v8=v8(find(v8>lo_wn)); 
v9=v9(find(v9>lo_wn)); v10=v10(find(v10>lo_wn)); v11=v11(find(v11>lo_wn)); v12=v12(find(v12>lo_wn)); 

Rp_000=Rp_000(find(v1<up_wn));       Rs_000=Rs_000(find(v2<up_wn));        Rp_4500=Rp_4500(find(v3<up_wn));      Rs_4500=Rs_4500(find(v4<up_wn)); 
Rp_0900=Rp_0900(find(v5<up_wn));     Rs_0900=Rs_0900(find(v6<up_wn));      Rp_09045=Rp_09045(find(v7<up_wn));    Rs_09045=Rs_09045(find(v8<up_wn));
Rp_909090=Rp_909090(find(v9<up_wn)); Rs_909090=Rs_909090(find(v10<up_wn)); Rp_909045=Rp_909045(find(v11<up_wn)); Rs_909045=Rs_909045(find(v12<up_wn));

v1=v1(find(v1<up_wn)); v2=v2(find(v2<up_wn));    v3=v3(find(v3<up_wn));    v4=v4(find(v4<up_wn)); 
v5=v5(find(v5<up_wn)); v6=v6(find(v6<up_wn));    v7=v7(find(v7<up_wn));    v8=v8(find(v8<up_wn));
v9=v9(find(v9<up_wn)); v10=v10(find(v10<up_wn)); v11=v11(find(v11<up_wn)); v12=v12(find(v12<up_wn));
%
% Reduce data point to save time. (by Cheng)
new_v=(lo_wn:10:up_wn)';

Rp_000=interp1(v1,Rp_000,new_v,'pchip');         Rs_000=interp1(v1,Rs_000,new_v,'pchip');
Rp_4500=interp1(v1,Rp_4500,new_v,'pchip');       Rs_4500=interp1(v1,Rs_4500,new_v,'pchip');
Rp_0900=interp1(v1,Rp_0900,new_v,'pchip');       Rs_0900=interp1(v1,Rs_0900,new_v,'pchip');
Rp_09045=interp1(v1,Rp_09045,new_v,'pchip');     Rs_09045=interp1(v1,Rs_09045,new_v,'pchip');
Rp_909090=interp1(v1,Rp_909090,new_v,'pchip');   Rs_909090=interp1(v1,Rs_909090,new_v,'pchip');
Rp_909045=interp1(v1,Rp_909045,new_v,'pchip');   Rs_909045=interp1(v1,Rs_909045,new_v,'pchip');

v1=new_v; v2=new_v; v3=new_v; v4=new_v; v5=new_v; v6=new_v; v7=new_v; v8=new_v; v9=new_v; v10=new_v; v11=new_v; v12=new_v;
%}
% Change wavenumber vector direction for each reflectance spectrum and
% combine into one array.
v1=v1(:); v2=v2(:);   v3=v3(:);   v4=v4(:);
v5=v5(:); v6=v6(:);   v7=v7(:);   v8=v8(:);
v9=v9(:); v10=v10(:); v11=v11(:); v12=v12(:);

v=cat(1,v1,v2,v3,v4,v5,v6,v7,v8,v9,v10,v11,v12);

% Polarization angles with respect to the "lab" X-axis of the crystal for each 
% reflectance spectrum.
omega_1=0; omega_2=0; omega_3=45; omega_4=45; omega_5=0; omega_6=0; omega_7=0; omega_8=0; omega_9=90; omega_10=90; omega_11=90; omega_12=90;
chi_1=0;   chi_2=0;   chi_3=0;   chi_4=0;   chi_5=90;  chi_6=90;  chi_7=90;  chi_8=90;  chi_9=90;  chi_10=90;   chi_11=90; chi_12=90;
psi_1=0;   psi_2=0;   psi_3=0;  psi_4=0;  psi_5=0;   psi_6=0;   psi_7=45;  psi_8=45;  psi_9=90;  psi_10=90;   psi_11=45; psi_12=45;

% s or p polarised [s p]
pol_1=[0 1]; pol_2=[1 0]; pol_3=[0 1]; pol_4=[1 0];
pol_5=[0 1]; pol_6=[1 0]; pol_7=[0 1]; pol_8=[1 0];
pol_9=[0 1]; pol_10=[1 0]; pol_11=[0 1]; pol_12=[1 0];

% Incidence angle
global incidence_angle
incidence_angle=30;

%extend omega into a vector with length of each reflectance spectrum and
%combine into one array
omega_1=repmat(omega_1,length(v1),1); chi_1=repmat(chi_1,length(v1),1); psi_1=repmat(psi_1,length(v1),1); pol_1=repmat(pol_1,length(v1),1);
omega_2=repmat(omega_2,length(v2),1); chi_2=repmat(chi_2,length(v2),1); psi_2=repmat(psi_2,length(v2),1); pol_2=repmat(pol_2,length(v2),1);
omega_3=repmat(omega_3,length(v3),1); chi_3=repmat(chi_3,length(v3),1); psi_3=repmat(psi_3,length(v3),1); pol_3=repmat(pol_3,length(v3),1);
omega_4=repmat(omega_4,length(v4),1); chi_4=repmat(chi_4,length(v4),1); psi_4=repmat(psi_4,length(v4),1); pol_4=repmat(pol_4,length(v4),1);
omega_5=repmat(omega_5,length(v5),1); chi_5=repmat(chi_5,length(v5),1); psi_5=repmat(psi_5,length(v5),1); pol_5=repmat(pol_5,length(v5),1); 
omega_6=repmat(omega_6,length(v6),1); chi_6=repmat(chi_6,length(v6),1); psi_6=repmat(psi_6,length(v6),1); pol_6=repmat(pol_6,length(v6),1);
omega_7=repmat(omega_7,length(v7),1); chi_7=repmat(chi_7,length(v7),1); psi_7=repmat(psi_7,length(v7),1); pol_7=repmat(pol_7,length(v7),1);
omega_8=repmat(omega_8,length(v8),1); chi_8=repmat(chi_8,length(v8),1); psi_8=repmat(psi_8,length(v8),1); pol_8=repmat(pol_8,length(v8),1);
omega_9=repmat(omega_9,length(v9),1); chi_9=repmat(chi_9,length(v9),1); psi_9=repmat(psi_9,length(v9),1); pol_9=repmat(pol_9,length(v9),1);
omega_10=repmat(omega_10,length(v10),1); chi_10=repmat(chi_10,length(v10),1); psi_10=repmat(psi_10,length(v10),1); pol_10=repmat(pol_10,length(v10),1);
omega_11=repmat(omega_11,length(v11),1); chi_11=repmat(chi_11,length(v11),1); psi_11=repmat(psi_11,length(v11),1); pol_11=repmat(pol_11,length(v11),1);
omega_12=repmat(omega_12,length(v12),1); chi_12=repmat(chi_12,length(v12),1); psi_12=repmat(psi_12,length(v12),1); pol_12=repmat(pol_12,length(v12),1);

omega=cat(1,omega_1,omega_2,omega_3,omega_4,omega_5,omega_6,omega_7,omega_8,omega_9,omega_10,omega_11,omega_12);
chi=cat(1,chi_1,chi_2,chi_3,chi_4,chi_5,chi_6,chi_7,chi_8,chi_9,chi_10,chi_11,chi_12);
psi=cat(1,psi_1,psi_2,psi_3,psi_4,psi_5,psi_6,psi_7,psi_8,psi_9,psi_10,psi_11,psi_12);
pol=cat(1,pol_1,pol_2,pol_3,pol_4,pol_5,pol_6,pol_7,pol_8,pol_9,pol_10,pol_11,pol_12);

% Create "predictor" array for fitting the reflectance spectrum
p1=cat(2,v1,omega_1,chi_1,psi_1,pol_1); p2=cat(2,v2,omega_2,chi_2,psi_2,pol_2); p3=cat(2,v3,omega_3,chi_3,psi_3,pol_3); p4=cat(2,v4,omega_4,chi_4,psi_4,pol_4);
p5=cat(2,v5,omega_5,chi_5,psi_5,pol_5); p6=cat(2,v6,omega_6,chi_6,psi_6,pol_6); p7=cat(2,v7,omega_7,chi_7,psi_7,pol_7); p8=cat(2,v8,omega_8,chi_8,psi_8,pol_8);
p9=cat(2,v9,omega_9,chi_9,psi_9,pol_9); p10=cat(2,v10,omega_10,chi_10,psi_10,pol_10); p11=cat(2,v11,omega_11,chi_11,psi_11,pol_11); p12=cat(2,v12,omega_12,chi_12,psi_12,pol_12);
p=cat(2,v,omega,chi,psi,pol);

% Convert to reflectance from percent reflectance
r=cat(1, Rp_000, Rs_000, Rp_4500, Rs_4500, ...
         Rp_0900, Rs_0900, Rp_09045, Rs_09045,...
         Rp_909090, Rs_909090, Rp_909045, Rs_909045);


%% Section 2
% Read in parameter estimates, upper and lower bounds from 
% file param_values.txt
%{
%get number of oscillators
fid=fopen('./param_values.txt');
ncols=length(textscan(fid,'','HeaderLines',1,'Delimiter',','));
disp(['number of oscillators: ', num2str(ncols)]);
fclose(fid);

%read paramter input file
parameter_file = fopen('./param_values.txt'); 
%read in parameter estimates
textformat=repmat('%f',1,ncols);
nu=textscan(parameter_file,textformat,'HeaderLines',2,'Delimiter',',');
gamm = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
Sk = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
phi = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
theta = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
textformat='%f';
epsilxx = textscan(parameter_file,textformat,'HeaderLines',1); 
epsilxy = textscan(parameter_file,textformat,'HeaderLines',1);
epsilxz = textscan(parameter_file,textformat,'HeaderLines',1);
epsilyy = textscan(parameter_file,textformat,'HeaderLines',1);
epsilyz = textscan(parameter_file,textformat,'HeaderLines',1);
epsilzz = textscan(parameter_file,textformat,'HeaderLines',1);
%read in lower bounds
textformat=repmat('%f',1,ncols);
lb_nu=textscan(parameter_file,textformat,'HeaderLines',2,'Delimiter',',');
lb_gamm = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
lb_Sk = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
lb_phi = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
lb_theta = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
textformat='%f';
lb_epsilxx = textscan(parameter_file,textformat,'HeaderLines',1); 
lb_epsilxy = textscan(parameter_file,textformat,'HeaderLines',1);
lb_epsilxz = textscan(parameter_file,textformat,'HeaderLines',1);
lb_epsilyy = textscan(parameter_file,textformat,'HeaderLines',1);
lb_epsilyz = textscan(parameter_file,textformat,'HeaderLines',1);
lb_epsilzz = textscan(parameter_file,textformat,'HeaderLines',1);
%read in upper bounds
textformat=repmat('%f',1,ncols);
ub_nu=textscan(parameter_file,textformat,'HeaderLines',2,'Delimiter',',');
ub_gamm = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
ub_Sk = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
ub_phi = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
ub_theta = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
textformat='%f';
ub_epsilxx = textscan(parameter_file,textformat,'HeaderLines',1); 
ub_epsilxy = textscan(parameter_file,textformat,'HeaderLines',1);
ub_epsilxz = textscan(parameter_file,textformat,'HeaderLines',1);
ub_epsilyy = textscan(parameter_file,textformat,'HeaderLines',1);
ub_epsilyz = textscan(parameter_file,textformat,'HeaderLines',1);
ub_epsilzz = textscan(parameter_file,textformat,'HeaderLines',1);
fclose(parameter_file);

% Concatenate together guesses to make coefficient vector. lsqcurvefit
% requires the model parameters to be in one vector or rectangular array.
% The user may wan to edit the lsqcurvefit function to accept more than one
% set of cofficients, eliminating the wrapper function.
coefg=cell2mat([nu,gamm,Sk,phi,theta,epsilxx,epsilxy,epsilxz,epsilyy,epsilyz,epsilzz]);

% Concatenate together upper and lower bounds.
lb=cell2mat([lb_nu,lb_gamm,lb_Sk,lb_phi,lb_theta,lb_epsilxx,lb_epsilxy,lb_epsilxz,lb_epsilyy,lb_epsilyz,lb_epsilzz]);
ub=cell2mat([ub_nu,ub_gamm,ub_Sk,ub_phi,ub_theta,ub_epsilxx,ub_epsilxy,ub_epsilxz,ub_epsilyy,ub_epsilyz,ub_epsilzz]);
%}

%parameter estimates
%
param = importdata('20190814.csv');
numb = (length(param)-6)/5;
nu = param(1:numb);
gamm = param(numb+1:2*numb);
Sk = param(2*numb+1:3*numb);
phi = param(3*numb+1:4*numb);
theta = param(4*numb+1:5*numb);
epsilxx = param(5*numb+1);
epsilxy = param(5*numb+2);
epsilxz = param(5*numb+3);
epsilyy = param(5*numb+4);
epsilyz = param(5*numb+5);
epsilzz = param(5*numb+6);

%}
%{
nu      = [210,235,275,320,335,350,360,370,380,390,400,410,420,435,445,460,510,520,540,570,595,605,615,625,635,680,710,730,750,770,920,930,940,950,960,970,980,1000,1015,1045,1075,1100,1125,1178];
gamm    = repmat(50,length(nu),1)';
Sk      = repmat(5000,length(nu),1)';
phi     = zeros(length(nu),1)';
theta   = zeros(length(nu),1)';
epsilxx = 3; 
epsilxy = 0.1;
epsilxz = 0.1;
epsilyy = 3;
epsilyz = 0.1;
epsilzz = 3;
%}
%read in lower bounds
lb_nu      = nu-50;
%lb_nu      = repmat(200,length(nu),1)';
lb_gamm    = zeros(length(nu),1)';
lb_Sk      = repmat(1000,length(nu),1)';
lb_phi     = repmat(-180,length(nu),1)';
lb_theta   = repmat(-180,length(nu),1)';
lb_epsilxx = 1; 
lb_epsilxy = 0.001;
lb_epsilxz = 0.001;
lb_epsilyy = 1;
lb_epsilyz = 0.001;
lb_epsilzz = 1;
%read in upper bounds
ub_nu      = nu+50;
%ub_nu      = repmat(1230,length(nu),1)';
ub_gamm    = repmat(200,length(nu),1)';
ub_Sk      = repmat(1000000,length(nu),1)';
ub_phi     = repmat(180,length(nu),1)';
ub_theta   = repmat(180,length(nu),1)';
ub_epsilxx = 5; 
ub_epsilxy = 1;
ub_epsilxz = 1;
ub_epsilyy = 5;
ub_epsilyz = 1;
ub_epsilzz = 5;

% Concatenate together guesses to make coefficient vector. 
% lsqcurvefit requires the model parameters to be in one vector or rectangular array.
% The user may wan to edit the lsqcurvefit function to accept more than one
% set of cofficients, eliminating the wrapper function.
coefg=cat(2,nu,gamm,Sk,phi,theta,epsilxx,epsilxy,epsilxz,epsilyy,epsilyz,epsilzz);

% Concatenate together upper and lower bounds.
lb=cat(2,lb_nu,lb_gamm,lb_Sk,lb_phi,lb_theta,lb_epsilxx,lb_epsilxy,lb_epsilxz,lb_epsilyy,lb_epsilyz,lb_epsilzz);
ub=cat(2,ub_nu,ub_gamm,ub_Sk,ub_phi,ub_theta,ub_epsilxx,ub_epsilxy,ub_epsilxz,ub_epsilyy,ub_epsilyz,ub_epsilzz);

%get number of oscillators
disp(['number of oscillators: ', num2str(length(nu))]);

% Allows the user to see if there is a mis-match between the number of
% parameters and the number of bounds
if(or(length(coefg)~=length(lb),length(coefg)~=length(ub)))
    display('Number of coefficients do not match number of bounds!');
end
disp(['number of coefficients: ', num2str(length(coefg))]); 
disp(['number of lower bounds: ', num2str(length(lb))]); 
disp(['number of upper bounds: ', num2str(length(ub))]);
disp('Start:');
disp(datetime);

%% Section 3
% Do nonlinear fit of the reflectance spectra using lsqcurefit

disp('Fitting reflectance spectra...');
%{
options=optimset('Display','iter','MaxIter',30,'MaxFunEvals',1000000,...
    'TolX',0.000000001,'TolFun',0.000000001); %First 2 options display fit iterations. 
                                                                     %This can be removed to save time. If 
                                                                     %fit doesn't converge try increasing
                                                                     %MaxIter
[coeff,resnorm,res,exitflag,output,lambda,J]=lsqcurvefit(@disp_model_wrap_triclinic,coefg,p,r,lb,ub,options); 
%}
myObjFcn = @(coefg,p)disp_model_wrap_triclinic(coefg,p);
options=optimoptions(@lsqcurvefit,'Algorithm','trust-region-reflective',...
    'Display','iter','MaxIter',30,'MaxFunEvals',1000000, 'TolFun',0.0000001,'TolX', 0.0000001);
problem = createOptimProblem('lsqcurvefit','x0',coefg,'objective',myObjFcn,...
    'xdata',p,'ydata',r,'lb',lb,'ub',ub,'options',options);
ms = MultiStart;   
[coeff,fval,exitflag,output,solutions] = run(ms,problem,5);
%}
%% Section 4
% Produce final model values, output text files of optical constants and
% final parameter valuies.

% Run forward model to produce guess and fitted spectra r_omega_guess and 
% r_omega_final. Create output text files of optical constants and final 
% oscillator parameter values.
disp('Producing final fits...');

L=length(v1);
%{
[r_guess,n1g,n2g,n3g,k1g,k2g,k3g]=disp_model_wrap_triclinic(coefg,p);
Rp_000_guess=r_guess(1:L,1);
Rs_000_guess=r_guess(L+(1:L),1);
Rp_4500_guess=r_guess(2.*L+(1:L),1);
Rs_4500_guess=r_guess(3.*L+(1:L),1);
Rp_0900_guess=r_guess(4.*L+(1:L),1);
Rs_0900_guess=r_guess(5.*L+(1:L),1);
Rp_09045_guess=r_guess(6.*L+(1:L),1);
Rs_09045_guess=r_guess(7.*L+(1:L),1);
Rp_909090_guess=r_guess(8.*L+(1:L),1);
Rs_909090_guess=r_guess(9.*L+(1:L),1);
Rp_909045_guess=r_guess(10.*L+(1:L),1);
Rs_909045_guess=r_guess(11.*L+(1:L),1);
n1g=n1g(1:L); n2g=n2g(1:L); n3g=n3g(1:L);
k1g=k1g(1:L); k2g=k2g(1:L); n3g=n3g(1:L);
%}
[r_final,n1,n2,n3,k1,k2,k3]=disp_model_wrap_triclinic(coeff,p);
Rp_000_final=r_final(1:L,1);
Rs_000_final=r_final(L+(1:L),1);
Rp_4500_final=r_final(2.*L+(1:L),1);
Rs_4500_final=r_final(3.*L+(1:L),1);
Rp_0900_final=r_final(4.*L+(1:L),1);
Rs_0900_final=r_final(5.*L+(1:L),1);
Rp_09045_final=r_final(6.*L+(1:L),1);
Rs_09045_final=r_final(7.*L+(1:L),1);
Rp_909090_final=r_final(8.*L+(1:L),1);
Rs_909090_final=r_final(9.*L+(1:L),1);
Rp_909045_final=r_final(10.*L+(1:L),1);
Rs_909045_final=r_final(11.*L+(1:L),1);
n1=n1(1:L); n2=n2(1:L); n3=n3(1:L);
k1=k1(1:L); k2=k2(1:L); k3=k3(1:L);
%
% Calculae confidence bouds for relectance fit
[rpred,delta] = nlpredci(@disp_model_wrap_triclinic,p,coeff,res,'Jacobian',J+1e-6);
lower_r = rpred - delta;
upper_r = rpred + delta;

Rp_000_lower=lower_r(1:L,1);
Rs_000_lower=lower_r(L+(1:L),1);
Rp_4500_lower=lower_r(2.*L+(1:L),1);
Rs_4500_lower=lower_r(3.*L+(1:L),1);
Rp_0900_lower=lower_r(4.*L+(1:L),1);
Rs_0900_lower=lower_r(5.*L+(1:L),1);
Rp_09045_lower=lower_r(6.*L+(1:L),1);
Rs_09045_lower=lower_r(7.*L+(1:L),1);
Rp_909090_lower=lower_r(8.*L+(1:L),1);
Rs_909090_lower=lower_r(9.*L+(1:L),1);
Rp_909045_lower=lower_r(10.*L+(1:L),1);
Rs_909045_lower=lower_r(11.*L+(1:L),1);

Rp_000_upper=upper_r(1:L,1);
Rs_000_upper=upper_r(L+(1:L),1);
Rp_4500_upper=upper_r(2.*L+(1:L),1);
Rs_4500_upper=upper_r(3.*L+(1:L),1);
Rp_0900_upper=upper_r(4.*L+(1:L),1);
Rs_0900_upper=upper_r(5.*L+(1:L),1);
Rp_09045_upper=upper_r(6.*L+(1:L),1);
Rs_09045_upper=upper_r(7.*L+(1:L),1);
Rp_909090_upper=upper_r(8.*L+(1:L),1);
Rs_909090_upper=upper_r(9.*L+(1:L),1);
Rp_909045_upper=upper_r(10.*L+(1:L),1);
Rs_909045_upper=upper_r(11.*L+(1:L),1);
%}
%{
% write final oscillator parameters to file 'new_param_values.txt')
fid = fopen('param_values.txt','r');
i = 1;
tline = fgetl(fid);
temp_params{i} = tline;
while ischar(tline)
    i = i+1;
    tline = fgetl(fid);
    temp_params{i} = tline;
end
fclose(fid);
textformat=(repmat('%f,',1,ncols)); textformat=textformat(1:length(textformat)-1);
temp_params{3}=sprintf(textformat,coeff(1:ncols));
temp_params{5}=sprintf(textformat,coeff(ncols+(1:ncols)));
temp_params{7}=sprintf(textformat,coeff(2.*ncols+(1:ncols)));
temp_params{9}=sprintf(textformat,coeff(3.*ncols+(1:ncols)));
temp_params{11}=sprintf(textformat,coeff(4.*ncols+(1:ncols)));
temp_params{13}=sprintf('%f',coeff(5.*ncols+1));
temp_params{15}=sprintf('%f',coeff(5.*ncols+2));
temp_params{17}=sprintf('%f',coeff(5.*ncols+3));
temp_params{19}=sprintf('%f',coeff(5.*ncols+4));
temp_params{21}=sprintf('%f',coeff(5.*ncols+5));
temp_params{23}=sprintf('%f',coeff(5.*ncols+6));

fid = fopen('new_param_values19.txt', 'w');
fprintf(fid, '%s\n', temp_params{:});
fclose(fid);

% write final optical constants to a .txt file
opconst=cat(2,v1,n1,n2,n3,k1,k2,k3);
dlmwrite('nk_GemmyFeldspar_run19.txt', opconst, 'delimiter', '\t', 'precision', 6);

% write initial and final fits to .txt file
initial_ref=cat(2,v1,Rp_000, Rs_000, Rp_4500, Rs_4500, Rp_0900, Rs_0900, Rp_09045, Rs_09045, Rp_909090, Rs_909090, Rp_909045, Rs_909045);
dlmwrite('em1_GF_run19.txt', initial_ref, 'delimiter', '\t', 'precision',6);
final_ref=cat(2,v1,Rp_000_final, Rs_000_final, Rp_4500_final, Rs_4500_final, Rp_0900_final, Rs_0900_final, Rp_09045_final, Rs_09045_final, Rp_909090_final, Rs_909090_final, Rp_909045_final, Rs_909045_final);
dlmwrite('emf_GF_run19.txt', final_ref, 'delimiter', '\t', 'precision',6);
%}
%poolobj = gcp('nocreate');
%delete(poolobj); %close any remaining pools


%% Section 5
% Plot results!
disp('Plotting results...');
%{ 
% Plot modeled and observed reflectance spectra on the same axes.
figure(1); clf;
set(0,'DefaultAxesColorOrder',[1 0 0;0 0 0],'DefaultAxesLineStyleOrder','-')
subplot(6,2,1); plot(v1,Rp_000,'k',v1,Rp_000_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rp 000','modeled','Location','northwest')
subplot(6,2,2); plot(v2,Rs_000,'k',v2,Rs_000_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rs 000','modeled','Location','northwest')
 
subplot(6,2,3); plot(v3,Rp_4500,'k',v3,Rp_4500_final,'r','LineWidth',2.0);
xlim([200 2000]); ylim([0 1]); set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rp 4500','modeled','Location','northwest')
subplot(6,2,4); plot(v4,Rs_4500,'k',v4,Rs_4500_final,'r','LineWidth',2.0);
xlim([200 2000]); ylim([0 1]); set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rs 4500','modeled','Location','northwest')
 
subplot(6,2,5); plot(v5,Rp_0900,'k',v5,Rp_0900_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rp 0900','modeled','Location','northwest')
subplot(6,2,6); plot(v6,Rs_0900,'k',v6,Rs_0900_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rs 0900','modeled','Location','northwest')
 
subplot(6,2,7); plot(v7,Rp_09045,'k',v7,Rp_09045_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rp 09045','modeled','Location','northwest')
subplot(6,2,8); plot(v8,Rs_09045,'k',v8,Rs_09045_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rs 09045','modeled','Location','northwest')
 
subplot(6,2,9); plot(v9,Rp_909090,'k',v9,Rp_909090_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rp 909090','modeled','Location','northwest')
subplot(6,2,10); plot(v10,Rs_909090,'k',v10,Rs_909090_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rs 909090','modeled','Location','northwest')
 
subplot(6,2,11); plot(v11,Rp_909045,'k',v11,Rp_909045_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rp_909045','modeled','Location','northwest')
xlabel('Wavenumber (cm^{-1})','FontSize',14)
subplot(6,2,12); plot(v12,Rs_909045,'k',v12,Rs_909045_final,'r','LineWidth',2.0);
xlim([200 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('Rs 909045','modeled','Location','northwest')
xlabel('Wavenumber (cm^{-1})','FontSize',14)
%}
%{ 
% Plot initial guess and observed reflectance on the same axes.
figure(2); clf;
set(0,'DefaultAxesColorOrder',[0 .5 .5;.5 0 .5],'DefaultAxesLineStyleOrder','-')
subplot(6,2,1); plot(v1,Rp_000_guess,v1,Rp_000,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rp 000','Location','northwest')
subplot(6,2,2); plot(v2,Rs_000_guess,v2,Rs_000,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rs 000','Location','northwest')
 
subplot(6,2,3); plot(v3,Rp_4500_guess,v3,Rp_4500,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rp 4500','Location','northwest')
subplot(6,2,4); plot(v4,Rs_4500_guess,v4,Rs_4500,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rs 4500','Location','northwest')
 
subplot(6,2,5); plot(v5,Rp_0900_guess,v5,Rp_0900,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rp 0900','Location','northwest')
subplot(6,2,6); plot(v6,Rs_0900_guess,v6,Rs_0900,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rs 0900','Location','northwest')
 
subplot(6,2,7); plot(v7,Rp_09045_guess,v7,Rp_09045,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rp_09045','Location','northwest')
subplot(6,2,8); plot(v7,Rs_09045_guess,v7,Rs_09045,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rs 09045','Location','northwest')
 
subplot(6,2,9); plot(v9,Rp_909090_guess,v9,Rp_909090,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rp 909090','Location','northwest')
subplot(6,2,10); plot(v10,Rs_909090_guess,v10,Rs_909090,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rs 909090','Location','northwest')
 
subplot(6,2,11); plot(v11,Rp_909045_guess,v11,Rp_909045,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rp 909045','Location','northwest')
xlabel('Wavenumber (cm^{-1})','FontSize',14)
subplot(6,2,12); plot(v12,Rs_909045_guess,v12,Rs_909045,'LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('estimated','Rs 909045','Location','northwest')
xlabel('Wavenumber (cm^{-1})','FontSize',14)
%
% Plot residual difference between the modeled and measured reflectance.
figure(3); clf;
subplot(6,1,1); plot(v1,Rp_000-Rp_000_final,'-k',v2,Rs_000-Rs_000_final,'-k','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('Rp 000','Rs 000','Location','northwest')
subplot(6,1,2); plot(v3,Rp_4500-Rp_4500_final,'-k',v4,Rs_4500-Rs_4500_final,'-k','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('Rp 4500','Rs 4500','Location','northwest')
subplot(6,1,3); plot(v5,Rp_0900-Rp_0900_final,'-k',v6,Rs_0900-Rs_0900_final,'-k','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('Rp 0900','Rs 0900','Location','northwest')
subplot(6,1,4); plot(v7,Rp_09045-Rp_09045_final,'-k',v8,Rs_09045-Rs_09045_final,'-k','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('Rp 09045','Rs 09045','Location','northwest')
subplot(6,1,5); plot(v9,Rp_909090-Rp_909090_final,'-k',v10,Rs_909090-Rs_909090_final,'-k','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('Rp 909090','Rs 909090','Location','northwest')
subplot(6,1,6); plot(v11,Rp_909045-Rp_909045_final,'-k',v12,Rs_909045-Rs_909045_final,'-k','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('Rp 909045','Rs 909045','Location','northwest')
xlabel('Wavenumber (cm^{-1})','FontSize',14)

% Plot modeled n and k
figure(4); clf;
subplot(2,1,1); plot(v1,n1,'r-',v1,n2,'g-',v3,n3,'b-','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 5]);
set(gca,'XDir','reverse')
ylabel('n1, n2, n3');
xlabel('Wavenumber (cm^{-1})','FontSize',14);
legend('n1','n2','n3','Location','northwest');
subplot(2,1,2); semilogy(v1,abs(k1),'r-',v1,abs(k2),'g-',v1,abs(k3),'b-','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 5]);
set(gca,'XDir','reverse')
ylabel('k1, k2, k3');
xlabel('Wavenumber (cm^{-1})','FontSize',14);
legend('k1','k2','k3','Location','northwest');
 
%plot modeled reflectance curves w/ 95% ci
figure(5); clf;
subplot(6,2,1); plot(v1,Rp_000_upper,'g--',v1,Rp_000_final,'b-',v1,Rp_000_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rp 000','upper bound','Location','northwest')
subplot(6,2,2); plot(v2,Rs_000_upper,'g--',v2,Rs_000_final,'b-',v2,Rs_000_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rs 000','upper bound','Location','northwest')
 
subplot(6,2,3); plot(v3,Rp_4500_upper,'g--',v3,Rp_4500_final,'b-',v3,Rp_4500_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rp 4500','upper bound','Location','northwest')
subplot(6,2,4); plot(v4,Rs_4500_upper,'g--',v4,Rs_4500_final,'b-',v4,Rs_4500_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rs 4500','upper bound','Location','northwest')
 
subplot(6,2,5); plot(v5,Rp_0900_upper,'g--',v5,Rp_0900_final,'b-',v5,Rp_0900_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rp 0900','upper bound','Location','northwest')
subplot(6,2,6); plot(v6,Rs_0900_upper,'g--',v6,Rs_0900_final,'b-',v6,Rs_0900_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rs 0900','upper bound','Location','northwest')
 
subplot(6,2,7); plot(v7,Rp_09045_upper,'g--',v7,Rp_09045_final,'b-',v7,Rp_09045_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rp 09045','upper bound','Location','northwest')
subplot(6,2,8); plot(v7,Rs_09045_upper,'g--',v7,Rs_09045_final,'b-',v7,Rs_09045_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rs 09045','upper bound','Location','northwest')
 
subplot(6,2,9); plot(v9,Rp_909090_upper,'g--',v9,Rp_909090_final,'b-',v9,Rp_909090_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rp 909090','upper bound','Location','northwest')
subplot(6,2,10); plot(v10,Rs_909090_upper,'g--',v10,Rs_909090_final,'b-',v10,Rs_909090_lower,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rs 909090','upper bound','Location','northwest')
 
subplot(6,2,11); plot(v11,Rp_909045_final,'g--',v11,Rp_909045_final,'b-',v11,Rp_909045_final,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rp 909045','upper bound','Location','northwest')
xlabel('Wavenumber (cm^{-1})','FontSize',14)
subplot(6,2,12); plot(v12,Rs_909045_final,'g--',v12,Rs_909045_final,'b-',v12,Rs_909045_final,'g--','LineWidth',3.0);
xlim([lo_wn up_wn]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('lower bound','modeled Rs 909045','upper bound','Location','northwest')
xlabel('Wavenumber (cm^{-1})','FontSize',14)
%}
display('...All done!');
display(datetime);
csvwrite('param_temp_190814.csv',coeff);
%fval
